@@ -33,11 +33,29 @@ extern "C" {
 #endif
 
 #define BUILD_FLOAT(fprec) MACRO_BUILD_FLOAT(fprec)
+
+#if defined(_ENABLE_MP_FLOATS) && defined(_ENABLE_LDOUBLE_FLOATS)
 #define BUILD_FLOATS \
 	BUILD_FLOAT(FP_SINGLE) \
 	BUILD_FLOAT(FP_DOUBLE) \
 	BUILD_FLOAT(FP_LDOUBLE) \
 	BUILD_FLOAT(FP_MP)
+#elif defined(_ENABLE_MP_FLOATS) && !defined(_ENABLE_LDOUBLE_FLOATS)
+#define BUILD_FLOATS \
+	BUILD_FLOAT(FP_SINGLE) \
+	BUILD_FLOAT(FP_DOUBLE) \
+	BUILD_FLOAT(FP_MP)
+#elif !defined(_ENABLE_MP_FLOATS) && defined(_ENABLE_LDOUBLE_FLOATS)
+#define BUILD_FLOATS \
+	BUILD_FLOAT(FP_SINGLE) \
+	BUILD_FLOAT(FP_LDOUBLE) \
+	BUILD_FLOAT(FP_DOUBLE)
+#else
+#define BUILD_FLOATS \
+	BUILD_FLOAT(FP_SINGLE) \
+	BUILD_FLOAT(FP_DOUBLE)
+#endif
+
 #define MACRO_BUILD_FLOATS BUILD_FLOATS
 
 #ifdef __cplusplus
